/*
** graphics library code
** 6/22/09 changed SPI to prescal 1:1 --speed up
6/22/09 added Vmap array and clearDisplay function
9/26/09 adding bitmap processing
7/18/10  changes for 102-6
modified for PIC32 version 0
mosidifed for PIC32 version 1
*********************************************
thk 5/1/2011
adapted for LCDBlocking Interface
extern BYTE LCDText[16*2+1];
void LCDInit(void);
void LCDUpdate(void);
void LCDErase(void);
void LCDWriteLine(WORD number, char *line);

*/


/* MPLAB includes. */
#include <p32xxxx.h>
#include <plib.h>
#include "graphics.h"
#include "Delay.h"

#include "font.h"
#define REDBACKLT _LATB3  //red blacklight control

#define greenLED _LATD10  //on board green led
#define redLED _LATF4   //on board red led

void flipfont(const char *pf);

int cx, cy;
	unsigned char tempfont=0;
	
/// LCDText is a 32 byte shadow of the LCD text.  Write to it and 
// then call LCDUpdate() to copy the string into the LCD module.
// orginally a 16x2 dispay with 102x64 graphics an 8x8 font
//we should have 12x8
//BYTE LCDText[16*2+1];
unsigned char LCDText[12*8+1];

//int YIN[128]; ***change for 102-6
int YIN[102];
unsigned char ramfont[8];



// I/O definitions

//#define CS   _LATB2    // CS output to EADOG128
//#define RESET  _LATB3   // RESET output EADOG128
//#define A0   _LATB5   // Ao output to EADOG128

#define CS   _LATB0    // CS output to EADOG102
#define RESET  _LATB1   // RESET output EADOG102
#define A0   _LATB2   // Ao output to EADOG102 CMD/data

const char init[] = {0x40,    //Display start line 0 (6)
                     0xa1,    //SEG reverse (13)
                     0xc0,    //Normal COM0...COM63 (14)
                     0xa4,	  // disable -> set all pixel to ON(10)
                     0xa6,    //Display inverse off (11)
                     0xa2,    //Set Bias 1/9 (Duty 1/65) (17)
                     0x2f,    //Booster, Regulator and Follower On (5)
                     0x27,    //set constrast (8)
                     0x81,	  // set volume (9)
                     0x10,    //
                     0xfa,	  // set temp compensation (25)
                     0x90,
                     0xaf,    //Display on (12)
                     };
#define X0 0
#define Y0 (VRES/2)


int xus;
int yus;
 float xf;
float  yf;

unsigned char  Vmap[HRES][VRES/8];  // array 102 bytes by 8 bytes or 816 bytes ***change 102-6
//******************************************************************************************
//LCDBlocking Interface 5/1/2011-all functions follow
/**********************************************************
void LCDInit(void);
   LCDText[] is blanked,  SPI2 is initialized and Vmap is cleared and dumped to LCD
 */
void LCDInit(void){
initSD();
int i =0;
//zero out LCDText with blanks
for (i=0;i<sizeof LCDText;i++) 
	{
 		LCDText[i] = ' ';	
	}
	//set text cursor to 0,0		
AT( 0, 0)	
}

 /***********************************************
 void LCDUpdate(void);
 *Overview:        Copies the contents of the local LCDText[] array into the 
 *					LCD's internal display buffer.  Null terminators in 
 *					LCDText[] terminate the current line, so strings may be 
 *					printed directly to LCDText[]
 * we have 102x64 pixels that yields with 8x8 font a 12 columns x 8 rows or 96 character display
 */	
void LCDUpdate(void){
	BYTE i, j;
	//first line
	AT (0,0)
	for(i = 0; i < 12u; i++)
	{
		// Erase the rest of the line if a null char is 
		// encountered (good for printing strings directly)
		if(LCDText[i] == 0u)
		{
			for(j=i; j < 12u; j++)
			{
				LCDText[j] = ' ';
			}
		}
		putcV((int) LCDText[i]);
	
	} //end firstline

 //second line
//	AT (0,8)
	for(i = 12; i < 24u; i++)
	{
		// Erase the rest of the line if a null char is 
		// encountered (good for printing strings directly)
		if(LCDText[i] == 0u)
		{
			for(j=i; j < 24u; j++)
			{
				LCDText[j] = ' ';
			}
		}
		putcV((int) LCDText[i]);
	
	} //end secondline
	 dumpVmap();
 //third line
//	AT (0,16)
	for(i = 24; i < 36u; i++)
	{
		// Erase the rest of the line if a null char is 
		// encountered (good for printing strings directly)
		if(LCDText[i] == 0u)
		{
			for(j=i; j < 36u; j++)
			{
				LCDText[j] = ' ';
			}
		}
		putcV((int) LCDText[i]);
	
	} //end thirdline	
 //fourth line
//	AT (0,24)
	for(i = 36; i < 48u; i++)
	{
		// Erase the rest of the line if a null char is 
		// encountered (good for printing strings directly)
		if(LCDText[i] == 0u)
		{
			for(j=i; j < 48u; j++)
			{
				LCDText[j] = ' ';
			}
		}
		putcV((int) LCDText[i]);
	
	} //end fourth line	
 //fifth line
//	AT (0,32)
	for(i = 48; i < 60u; i++)
	{
		// Erase the rest of the line if a null char is 
		// encountered (good for printing strings directly)
		if(LCDText[i] == 0u)
		{
			for(j=i; j < 60u; j++)
			{
				LCDText[j] = ' ';
			}
		}
		putcV((int) LCDText[i]);
	
	} //end fifth line
//sixth line
//	AT (0,40)
	for(i = 60; i < 72u; i++)
	{
		// Erase the rest of the line if a null char is 
		// encountered (good for printing strings directly)
		if(LCDText[i] == 0u)
		{
			for(j=i; j < 72u; j++)
			{
				LCDText[j] = ' ';
			}
		}
		putcV((int) LCDText[i]);
	
	} //end sixth line	
//seventh line
//	AT (0,48)
	for(i = 72; i < 84u; i++)
	{
		// Erase the rest of the line if a null char is 
		// encountered (good for printing strings directly)
		if(LCDText[i] == 0u)
		{
			for(j=i; j < 84u; j++)
			{
				LCDText[j] = ' ';
			}
		}
		putcV((int) LCDText[i]);
	
	} //end seventh line		
//eighth line
//	AT (0,56)
	for(i = 84; i < 96u; i++)
	{
		// Erase the rest of the line if a null char is 
		// encountered (good for printing strings directly)
		if(LCDText[i] == 0u)
		{
			for(j=i; j < 96u; j++)
			{
				LCDText[j] = ' ';
			}
		}
		putcV((int) LCDText[i]);
	
	} //end eighth line	
		dumpVmap();	 //dump entire vmap onto display
		Nop();
}
/******************************************************************************
 * Function:        void LCDErase(void)
 * Overview:        Clears LCDText[], and clearsscreen ( clears vmap and display)
 ****************************************************************************/ 	
void LCDErase(void){
	clearScreen();	
	
int i =0;
//zero out LCDText with blanks
for (i=0;i<sizeof LCDText;i++) 
	{
 		LCDText[i] = ' ';	
	}
	//set text cursor to 0,0		
AT( 0, 0)	
	
}	
//void LCDWriteLine(WORD number, char *line);
void LCDWriteLine(WORD number, char *line)
{ //number =row 1,2,3,4,5,6,7,8
    BYTE    i;
    BYTE    j;

    j = 0;
    i = 0;
        
    switch (number){
    case 1: 
    j=0;
    break;
    case 2:
    j=12;
    break;
    case 3:
    j=24;
    break;
    case 4:
    j=36;
    break;
    case 5:
    j=48;
    break;
    case 6:
    j=60;
    break;
    case 7:
    j=72;
    break;
    case 8:
    j=84;
    break;
    default:
    j=0;
    break;
}
    do 
    {
        LCDText[j++] = line[i++];
    }    
    while ((LCDText[j-1] != 0) && (j < 95));
    
    LCDUpdate();
}

void redBLON(void){
	
	REDBACKLT =1;
}	
void redBLOFF(void){
	REDBACKLT =0;
}
void greenBLOFF(void){
 SetDCOC5PWM(0); 	
	
}	
void greenBLON( int setting){
 SetDCOC5PWM(setting); 	
	
}	
void greenLEDON(void){
	greenLED =1;
}	
void greenLEDOFF(void){
	greenLED =0;
}	
void redLEDON(void){
redLED =1;	
	
}	
void redLEDOFF(void){
redLED =0;	
}		
	

//*****************************************************************************************
void initSD( void)
{
	AD1PCFG = 0xFFFFFFFF; //added for EXP 32
    DDPCON=0x00; //added fOR EXP 32 to turn off JTAG
   
    // added OC3 and Timer3 for PWM led green backlight
OpenTimer3(T3_ON |T3_PS_1_1, 0xFFFF);
OpenOC5(OC_ON | OC_TIMER3_SRC |OC_TIMER_MODE16| OC_PWM_FAULT_PIN_DISABLE, 0x7fff, 0x7fff);

    
   int i=0;
 
   	CS =1;
 
   A0=0;					//this must be equivalent of CD (=0) for command
 
   RESET =1;
    REDBACKLT =0;	//turn off backlight
   redLED =0;
   greenLED =0;
mPORTBSetPinsDigitalOut ( BIT_3|BIT_2 |BIT_1 | BIT_0 );
mPORTDSetPinsDigitalOut ( BIT_10); //red led
mPORTFSetPinsDigitalOut	(BIT_4);  //green led
	
    Delayms(50);  //10 msec
    RESET =0;
    Delayms(50); //10 msec
 
     RESET =1;
   
    // init the spi module for a slow (safe) clock speed first
OpenSPI2(SPI_MODE8_ON|CLK_POL_ACTIVE_LOW| ENABLE_SDO_PIN|SPI_CKE_ON|SPI_SMP_OFF|MASTER_ENABLE_ON|SEC_PRESCAL_1_1|PRI_PRESCAL_1_1,SPI_ENABLE);
SPI2BRG =15; //at PB =40 Mhz this is 1.25MHZ
//	SPI2CON1 = 0x013F;  // CKE=1; CKP=0, sample middle, prescale 1:1

//    SPI2STAT = 0x8000;  // enable the peripheral
     	CS =0;
    //initialize and enable the EADOG128
    
   
   
    for (i =0; i<sizeof init; i++)
    {
	    writeSPI(init[i]);
	    //delay_5us(); //not needed
	    
	   } 

	clearScreen();
redBLOFF();
      greenBLON(0xffff);
      redLEDON();
      greenLEDON();

			
	//draw a broken byte line stream				
	 //  goto_xy(0,3);
	//  	setAO();			//CD =1 or data
	//   for (i=0; i<64;i++)
	//   writeSPI(0x55);
	//   for (i=0; i<64;i++)
//	   writeSPI(0xaa);
}   // initSD

void clearScreen (void){
	
	clearVmap();
		dumpVmap();	
	
	
	
}	
// send one byte of data and receive one back at the same time
unsigned char writeSPI( unsigned char b)
{
    SPI2BUF = b;					// write to buffer for TX
    while( !SPI2STATbits.SPIRBF);	// wait for transfer to complete
 
    return SPI2BUF;    				// read the received value
}// writeSPI

void clearDisplay(void) {
	
	 //clear all display ram
	   int y=0;
	   int i=0;
	    for (y =0;y<8;y++)
	 {
	   goto_xy(0,y);
	   setAO();		//data mode
	   //	for (i=0; i<128;i++) ****change for 102-6
	   	 	for (i=0; i<102;i++)
	   	writeSPI(0x00); 
		}							

}
void clearVmap( void) {
	int y=0;
	int x=0;
	   // for (x =0;x<128;x++) //**change for 102-6
	    for (x =0;x<102;x++)
	    	for (y=0; y<8; y++)
	    	{
		    Vmap [x][y] =0;
				}
		    	
  }	
	
void dumpVmap( void)
{
	//dump Vmap onto display
	   int y=0;
	   int x=0;
	    for (y =0;y<8;y++)
	 {
	   goto_xy(0,y);
	   setAO();		//data mode
	 //  	for (x=0; x<128;x++) //****change for 102-6
	 	for (x=0; x<102;x++)
	   	writeSPI(Vmap[x][y]); 
		}				
	
	
}
//plots pixel point into processor ram x =0 to 102 y =0 to 63 ***change 102-6
void Plot (unsigned char x, unsigned y) 
{
	if ((x<HRES)&& (y<VRES)) {	
	unsigned char temp = 0;
	temp = Vmap[x][y/8];  //retrive byte
	temp =temp| (0x01 << (y%8))	; //or appropiate bit
	Vmap[x] [y/8] = temp;			//write back
	}
}			
	
	
		

// stuff lifted from EADOG driver*************************************

void goto_xy(char x, char y)
{
  char set[] = {0xb0, 0x10, 0x00};    //Befehlsarray
  set[0] = set[0] + y;                //Y - Position auf Steuerbefehl addieren
  set[2] = set[2] + (x & 0x0f);       //Low - Nibble auf Steuerbefehl addieren
  set[1] = set[1] + (x >> 4);         //High - Nibble auf Steuerbefehl addieren
 
   	clearA0();			//cmd mode
	writeSPI(set[0]);	//(7) page address
	writeSPI(set[2]);	// (4) low LSB first ***change for 102-6
	writeSPI(set[1]);	// (4) high MSB second *** this order is opposite to 128-6

}


//*************end of EADOG Driver**********************************

void enableCS(void) {
		CS =0;
	
}	

void disableCS(void) {
		CS =1;
	
}	
void setAO(void) {  //equivalent of CD command/data 1 =data
		A0 =1;
	
}	

void clearA0(void) {//equivalent of CD command/data 0 =command
		A0=0;
	
}

//Bresenham algorithm --line drawing algorithm uses only integers
#define abs( a)     (((a)> 0) ? (a) : -(a))

void line( int x0, int y0, int x1, int y1)
{
     int steep, t ;
     int deltax, deltay, error;
     int x, y;
     int ystep;

     steep = ( abs(y1 - y0) > abs(x1 - x0));
     
     if ( steep )
     { // swap x and y
         t = x0; x0 = y0; y0 = t;
         t = x1; x1 = y1; y1 = t;
     }
     if (x0 > x1) 
     {  // swap ends
         t = x0; x0 = x1; x1 = t;
         t = y0; y0 = y1; y1 = t;
     }
     
     deltax = x1 - x0;
     deltay = abs(y1 - y0);
     error = 0;
     y = y0;
     
     if (y0 < y1) ystep = 1; else ystep = -1;
     for (x = x0; x < x1; x++)
     {
         if ( steep) Plot(y,x); else Plot(x,y);
         error += deltay;
         if ( (error<<1) >= deltax)
         {
             y += ystep;
             error -= deltax;
         } // if 
     } // for
} // line 




void random_line_demo( void) {
 int i,x;
      srand( 12);
    for ( x=0; x<1; x++) {
         clearScreen();
       
         
         
        line( 0, 0, 0, VRES-1);
        line( 0, VRES-1, HRES-1, VRES-1);
        line( HRES-1, VRES-1, HRES-1, 0);
        line( 0, 0, HRES-1, 0);
        for( i = 0; i<50; i++) 
        {	
            line( rand()%HRES, rand()%VRES, rand()%HRES, rand()%VRES);
            
    		dumpVmap();
    			delay() ; //100s msec
								}
		}
			delay() ; //100s msec
				delay() ; //100s msec
					delay() ; //100s msec
					
}
void Rectangle ( int left, int top, int right, int bottom) {

	//draw rectangle one line at a time
	line( left,top, right,top);    //top line
	line ( left,bottom,right,bottom); //bottom line
	line (left,top, left,bottom);  // left side line
	line ( right,top, right, bottom); //right side line

	
	
}
void Bar ( int left, int top, int right, int bottom) {

	//draw rectangle one line at a time
	line( left,top, right,top);    //top line
	line ( left,bottom,right,bottom); //bottom line
	line (left,top, left,bottom);  // left side line
	line ( right,top, right, bottom); //right side line
		int y=0;
		int x=0;
	    for (x =0;x<128;x++)    //fill 
	    	for (y=0; y<64; y++)
	    	{
		    if ((( x>=left) && (x<=right)) && ((y <= bottom) && (y >=top)))
		    Plot(x,y);
				}
		    	
  }	
	



//void Circle ( int xcenter, int ycenter, int radius) { 
//
//	 int x, y, r2;
// 	r2 = radius * radius;
//	for (x = -radius; x <= radius; x++) {
//            y = (int) (sqrt(r2 - x*x) + 0.5);
//            Plot( xcenter + x, ycenter + y);
//            Plot( xcenter + x, ycenter - y);
//}
//}	
void rightTriangle ( int topx, int topy, int rightx, int righty) {

	//draw rectangle one line at a time
	line( topx,topy, rightx,righty );    //draw hypotenuse
	line ( topx,righty,topx,topy); //draw perpendicular
	line (topx,righty, rightx,righty);  // draw base
	
}

void Triangle ( int topx, int topy, int rightx, int righty) {
	int base =0;
	base = 2* rightx-topx;
	//draw rectangle one line at a time
	line( topx,topy, rightx,righty );    //draw hypotenuse
	//line ( topx,righty,topx,topy); //draw perpendicular
	line (topx-base/2,righty, rightx,righty);  // draw base
	line (topx-base/2, righty, topx,topy);// draw hypotenuse
	
}


//bit map handing rotuine for 32x32 pixels

void convertrow ( unsigned long tlong, unsigned char row,unsigned char  col ) 
{
	int m=0;
	unsigned char bitpos;
	unsigned char temp;
	if ( row < 8 ) {
	if (row ==0) bitpos =0x01;
	if (row ==1) bitpos =0x02;
	if (row ==2) bitpos =0x04;
	if (row ==3) bitpos =0x08;
	if (row ==4) bitpos =0x10;
	if (row ==5) bitpos =0x20;
	if (row ==6) bitpos =0x40;
	if (row ==7) bitpos =0x80;
	}
	else {
		
		temp = (row%8);
	 
	if (temp ==0) bitpos =0x01;
	if (temp==1) bitpos =0x02;
	if (temp ==2) bitpos =0x04;
	if (temp ==3) bitpos =0x08;
	if (temp ==4) bitpos =0x10;
	if (temp ==5) bitpos =0x20;
	if (temp ==6) bitpos =0x40;
	if (temp ==7) bitpos =0x80;
			
		
	}	

	
	unsigned long Lrotate = 0x000001;

	unsigned char tempvalue =0;
	for (m =0; m <32; m++) {
		tempvalue = Vmap[col+m][row/8]; 
	if ( (Lrotate & tlong) >=1)	{
	Vmap[col+m][row/8] = tempvalue | bitpos;	
			
	}		
	Lrotate =Lrotate <<1;

		
	}	

}	
void drawbitmap32x32 ( unsigned long *slp,unsigned char xoff, unsigned char yoff) {
	
	int m =0;
	unsigned char xplace = xoff;
	unsigned char yplace = yoff;
	
	long unsigned tlong;

	for ( m=0; m<32; m++) {
			tlong = *slp;

				//do domething with bytes to convert proper postion in VMAP
				convertrow (tlong, yplace,  xplace);  // take 32 bit word and set appropiate bit postion in 32 columns of display
				xplace =xoff;
				yplace++;
				slp++;
		
	}
	
	
	
}	

//text handling routines
void putcV( int a)
{
      int i;
   const char *pf;
//    // 1. check if char in range
    a -= F_OFFS;
    if ( a < 0)         a = 0;
    if ( a >= F_SIZE)   a = F_SIZE-1; 
     
    //zero ramfont
    
    for (i=0; i<7; i++) {
	  ramfont[i] =0 ;  
	   } 
    //int i, *p;  cx and cy are byte postions
  
    // 2. check page boundaries 
    if ( cx >= HRES/8)          // wrap around x
    {
        cx = 0;
        cy++;   
    } 
    if ( cy >= VRES/8)          // wrap around y (or scroll?)
        cy = 0;
        
  
   a= a<<3;
    pf = &Font8x8[a];
    flipfont( pf);
//      pf = &Font8x8[a];
    // 4. copy one by one each line of the character on the screen
    unsigned char temp;
    for ( i=0; i<8; i++)
    {

        temp = Vmap[cx*8+i][cy];
       // temp |= *pf++;
       temp =ramfont[7-i];  //found that foont is backwards--must adjust
        Vmap[cx*8+i][cy] =temp;
  
    } // for
    
    // increment cursor position
    cx++;
} // putcV

void putsV( unsigned char *s)
{
    while (*s)
        putcV( *s++);
} // putsV
//this is my attempt to flip rom font to ram font-still resutls in bit pattern backwards 
void flipfont(const char *pf)
{    

	unsigned char  rotate =0;
	unsigned char test =0;
	unsigned char fontbit =0;
	int i =0;
	int x =0;
	fontbit =0x01;
	for (i=0; i<8;i++)
	{
	tempfont = *pf++;
	rotate =0x01;
	
  	for (x=0; x<8; x++) 
  	{
	  test = tempfont & rotate;
	  if (test)
	  {
		  ramfont[x] =ramfont[x] |fontbit;
		 } 	
		
		 	rotate =rotate<<1;		 
	  	
	  }	
 fontbit =fontbit <<1;
 	
		
		
		
	}	
	






}
//new dot factory bit map handler 05/30/10
void drawbitmap ( unsigned char *pt, unsigned char xoff, unsigned char yoff, int width, int height) {
	
unsigned	 int tempb,tempt =0;
unsigned 	 int btest =0;


int i,h,w =0;
int x,y =0;

	for (h=0; h< height; h++) {  //height of h rows
		
	for (w=0; w< width; w++) {    // row of w bytes
		tempb = *pt;
		tempt =*pt;				// retreive byte
    for ( i =0; i<8; i++) {   //process one byte
	   
	    tempb = tempb << (1);
	    btest = tempb & 0x0100;
	    
	    if ( btest != 0) 
	    Plot( x+xoff, y+yoff);
	    x++;	    
	   } //end i 
		
	pt++;  //advance to next byte	
		
		
		
		
	} //end w		

	y++;
	x=0;	
		
		
		
		
		
	} //end h loop	
	
	
}	
